from typing import Any
from typing import Callable


def handle_result(data: Any, coerce_to: Callable = None, many: bool = False):
    if coerce_to is not None and not callable(coerce_to):
        raise ValueError(f"Callable expected in `coerce_to` got {coerce_to}")

    if coerce_to is None:
        return data

    if many is True:
        result = list(map(coerce_to, data))
    else:
        result = coerce_to(data)
    return result



from fibery.client.base import SyncClient
from fibery.conf import settings


def default_client(cls=SyncClient):
    return cls(token=settings.api_token.get_secret_value(), workspace=settings.workspace)



import asyncio

import typer

from fibery.conf import settings
from fibery.console.commands import pyppeteer_actions as action
from fibery.console.commands.base import output


token = typer.Typer()


@token.callback()
def token_callback(username: str = typer.Option(None), workspace: str = typer.Option(None)):
    settings.username = username or settings.username
    settings.workspace = workspace or settings.workspace


def run(coroutine):
    asyncio.get_event_loop().run_until_complete(coroutine)


def get_password():
    password = settings.password.get_secret_value()
    return password or typer.prompt("Password", hide_input=True)


@token.command(name="create")
def token_create():

    # Get password securely
    password = get_password()

    async def login_and_create_token():
        async with action.login(settings.username, password, settings.workspace, settings.login_url) as page:
            result = await action.do_create_token(page)
            output(result, headers=["Token created"]).echo()

    run(login_and_create_token())


@token.command(name="list")
def token_list():

    # Get password securely
    password = get_password()

    async def login_and_list_tokens():
        async with action.login(settings.username, password, settings.workspace, settings.login_url) as page:
            result = await action.do_list_tokens(page)
            output(result, headers="keys").echo()

    run(login_and_list_tokens())


@token.command(name="delete")
def token_delete(token_id: str = typer.Option(...)):

    # Get password securely
    password = get_password()

    async def login_and_delete_token():
        async with action.login(settings.username, password, settings.workspace, settings.login_url) as page:
            result = await action.do_delete_token(page, token_id)
            output(result).echo()

    run(login_and_delete_token())



import json
import os

from pathlib import Path
from typing import Any
from typing import Dict

from pydantic import BaseSettings
from pydantic import SecretStr

from fibery.enums import FormatEnum


def json_config(settings: "Settings") -> Dict[str, Any]:
    encoding = settings.__config__.env_file_encoding
    config_file = os.getenv("FIBERY_CONFIG", f"{Path.home()}/.fibery/config.json")
    return json.loads(Path(config_file).read_text(encoding))


class MaskedSecretStr(SecretStr):
    @property
    def masked(self) -> str:
        value = self.get_secret_value()
        last_chars = 4
        if len(value) < 8:
            last_chars = int(len(value) / 2) or 1
        return f'{"*" * (len(value) - last_chars)}{value[-last_chars:]}'

    def __str__(self):
        return self.masked


class Settings(BaseSettings):
    workspace: str = None
    username: str = None
    password: MaskedSecretStr = None
    api_token: MaskedSecretStr = None
    login_url: str = "https://fibery.io/login"
    output_format: FormatEnum = FormatEnum.text

    class Config:
        env_prefix = "FIBERY_"
        env_file_encoding = "utf-8"

        @classmethod
        def customise_sources(
            cls,
            init_settings,
            env_settings,
            file_secret_settings,
        ):
            return (
                init_settings,
                json_config,
                env_settings,
                file_secret_settings,
            )


settings = Settings()



import functools

import typer

from click import ClickException
from typer import Argument

from fibery.client.fibery import Fibery
from fibery.client.schema import FiberySchema
from fibery.console.commands.base import output
from fibery.console.commands.utils import default_client


describe = typer.Typer()


def get_schema() -> FiberySchema:
    fibery = Fibery(client=default_client())
    return fibery.schema.get()


def handle_exceptions(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        try:
            f(*args, **kwargs)
        except Exception as e:
            raise ClickException(str(e))

    return wrapper


@describe.command("applications")
@handle_exceptions
def list_applications():
    """
    List all Fibery apps on your account
    """
    schema = get_schema()
    result = [app.dict() for app in schema.list_applications()]
    for item in result:
        item["types"] = len(item["types"])
    output(result, headers="keys").echo()


@describe.command("types")
@handle_exceptions
def list_types(application: str = None):
    """
    List Fibery types on your account
    """
    schema = get_schema()
    result = []
    for fibery_type in schema.list_types(application):
        result.append(
            {
                "Name": fibery_type.name,
                "Fields number": len(fibery_type.fields),
            }
        )
    output(result, headers="keys").echo()


@describe.command("type")
@handle_exceptions
def get_type(name: str):
    """
    Describe single Fibery type with all attributes on your account
    """
    schema = get_schema()
    fibery_type = schema.get_type(name)
    result = fibery_type.dict()
    result.update({"fields": [f.name for f in fibery_type.fields]})
    output(result).echo()


@describe.command("fields")
@handle_exceptions
def list_fields(fibery_type: str = Argument(..., metavar="type")):
    """
    List Fibery all Type fields on your account
    """
    schema = get_schema()
    fibery_type = schema.get_type(fibery_type)
    result = []
    for field in fibery_type.fields:
        result.append(
            {
                "Name": field.name,
                "Type": field.type,
            }
        )
    output(result, headers="keys").echo()


@describe.command("field")
@handle_exceptions
def get_field(fibery_type: str = Argument(..., metavar="type"), name: str = Argument(...)):
    """
    Get single Field description on your account
    """
    schema = get_schema()
    fibery_type = schema.get_type(fibery_type)
    result = fibery_type.get_field(name)
    output(result).echo()


@describe.command(name="webhooks")
def list_webhooks():
    """
    List all registered hooks on your account
    """
    fibery = Fibery(client=default_client())
    webhooks = fibery.webhook.list()

    result = []
    for webhook in webhooks:
        result.append(webhook.dict())

    output(result, headers="keys", exclude=["runs"]).echo()



import os

from contextlib import asynccontextmanager

from click import ClickException
from pyppeteer import launch
from pyppeteer.page import Page


BASE_DIR = os.path.dirname(os.path.abspath(__file__))
JS_DIR = os.path.normpath(os.path.join(BASE_DIR, "js"))


@asynccontextmanager
async def login(
    username: str,
    password: str,
    workspace: str,
    login_url: str = "https://fibery.io/login",
    debug: bool = False,
):
    browser = await launch(headless=not debug)
    page = await browser.newPage()

    await do_login(page, login_url, username, password)
    await do_switch_workspace(page, workspace)

    yield page

    await browser.close()


def js(name):
    with open(os.path.join(JS_DIR, name + ".js"), "r") as h:
        return h.read()


async def do_login(page: Page, url: str, username: str, password: str):
    await page.goto(url, waitUntil="networkidle2")
    await page.waitForSelector("#email")

    # Input username
    await page.focus("#email")
    await page.keyboard.type(username)
    await page.focus("#password")
    await page.keyboard.type(password)
    await page.click("#ui-email-form > button")

    # Wait for workspaces loaded
    await page.waitForNavigation()


async def do_switch_workspace(page: Page, name: str):
    await page.goto(f"https://{name}.fibery.io", waitUntil="networkidle0")


async def do_create_token(page: Page):
    page_function = js("token_create")
    result = await page.evaluate(page_function)
    if "value" not in result:
        raise ClickException(f"Error while creating token: {result}")
    return result["value"]


async def do_list_tokens(page: Page):
    page_function = js("token_list")
    result = await page.evaluate(page_function)
    if not isinstance(result, list):
        raise ClickException(f"Error while getting tokens: {result}")
    return result


async def do_delete_token(page: Page, token_id: str):
    page_function = js("token_delete")
    page_function = page_function.replace("#token_id", token_id)
    result = await page.evaluate(page_function)
    if result is not True:
        raise ClickException(f"Error while deleting token: {result}")
    return f"Token with id={token_id} deleted"



from typing import Optional

import typer

from fibery.conf import settings
from fibery.console.commands.api import api
from fibery.console.commands.auth import auth
from fibery.console.commands.config import config
from fibery.console.commands.describe import describe
from fibery.enums import FormatEnum


main = typer.Typer()
main.add_typer(auth, name="auth")
main.add_typer(config, name="config")
main.add_typer(api, name="api")
main.add_typer(describe, name="describe")


@main.callback()
def main_callback(format: Optional[FormatEnum] = None):
    settings.output_format = format or settings.output_format



from typing import TYPE_CHECKING
from typing import Callable
from typing import Literal

import requests

from fibery.client.utils import handle_result


if TYPE_CHECKING:
    from fibery.client.fibery import Fibery


class ArgsDescriptor:
    pass


class FiberyApiError(Exception):
    pass


class BaseClient:

    workspace: str
    endpoint: str

    def __init__(self, token, workspace, endpoint="https://{workspace}.fibery.io/api/"):
        self.endpoint = endpoint.format(workspace=workspace)
        self.workspace = workspace
        self.token = token


class SyncClient(BaseClient):
    def request(
        self,
        method: Literal["GET", "POST", "PUT", "DELETE"],
        resource,
        json: dict = None,
        data: str = None,
        coerce_to: Callable = None,
        many=False,
    ):

        headers = {
            "Authorization": f"Token {self.token}",
            "Content-Type": "application/json",
            "Accept": "application/json",
        }

        url = "/".join([self.endpoint.strip("/"), resource])
        response = requests.request(method, url, json=json, data=data, headers=headers)
        if response.status_code != 200:
            raise FiberyApiError(f"Error {response.status_code}: {response.content}")

        result = response.json()
        return handle_result(result, coerce_to=coerce_to, many=many)

    def get(self, *args, **kwargs):
        return self.request("GET", *args, **kwargs)

    def update(self, *args, **kwargs):
        return self.request("PUT", *args, **kwargs)

    def create(self, *args, **kwargs):
        return self.request("POST", *args, **kwargs)

    def delete(self, *args, **kwargs):
        return self.request("DELETE", *args, **kwargs)

    def command(self, command, args=None, coerce_to: Callable = None, many=False):
        payload = {"command": command}
        if args:
            payload.update({"args": args})

        result = self.request("POST", "commands", json=[payload])[0]
        success = result["success"]
        result = result["result"]
        if not success:
            raise FiberyApiError(result)
        return handle_result(result, coerce_to=coerce_to, many=many)


class BaseResource:

    fibery: "Fibery" = None
    name: str = None

    def __init__(self, fibery):
        self.fibery = fibery

    @property
    def client(self):
        return self.fibery.client

    def get_methods(self):
        result = []
        for c in dir(self):
            member = getattr(self, c)
            if not callable(member):
                continue
            if c in {"init", "get_methods"}:
                continue
            if c.startswith("_"):
                continue
            result.append(c)

        return result



from typing import List
from typing import Optional

from pydantic import AnyUrl
from pydantic import BaseModel

from fibery.client.base import BaseResource
from fibery.models import FiberyEntityType


class Run(BaseModel):
    http_status: Optional[str]
    elapsed_time: Optional[str]
    request_time: Optional[str]


class AddResult(BaseModel):
    id: int
    url: AnyUrl
    type: FiberyEntityType
    state: str
    runs: List[Run] = []


class GetResult(AddResult):
    pass


class WebHook(BaseResource):

    name = "webhooks/v2"

    def create(self, url: str, type: FiberyEntityType) -> AddResult:
        return self.client.create(self.name, json={"url": url, "type": type}, coerce_to=AddResult.parse_obj)

    def list(self) -> List[GetResult]:
        return self.client.get(self.name, coerce_to=GetResult.parse_obj, many=True)

    def delete(self, pk) -> bool:
        name = f"{self.name}/{pk}"
        return self.client.delete(name)



import functools
import json

from typing import Dict
from typing import List
from typing import Literal
from typing import Optional
from typing import Union

import typer
import yaml

from pydantic import BaseModel
from tabulate import tabulate

from fibery.conf import settings


def handle_item(item, exclude: List[str] = None):
    for key in exclude or []:
        if isinstance(item, dict):
            item.pop(key, None)
    if settings.output_format == "text":
        for k, v in item.items():
            if isinstance(v, (list, dict)):
                item[k] = yaml.dump(v, Dumper=yaml.Dumper, default_style="", allow_unicode=True)
    return item


class Output:

    data: Union[BaseModel, Dict, List, str]
    headers: Union[Literal["keys", "firstrow"], List[str]] = None
    exclude: Optional[List[str]] = None

    def __init__(self, data, headers=None, exclude=None):
        if isinstance(data, list):
            data = list(map(functools.partial(handle_item, exclude=exclude), data))
        elif isinstance(data, BaseModel):
            data = data.dict()
            data = handle_item(data, exclude)
        elif isinstance(data, dict):
            data = handle_item(data, exclude)

        self.data = data
        self.headers = headers
        self.exclude = exclude

    def echo(self):
        try:
            getattr(self, f"output_{settings.output_format}")()
        except AttributeError:
            raise ValueError(f"{settings.output_format} is not implemented")

    def output_text(self):
        data = self.data
        if isinstance(data, str):
            data = [[data]]
        elif isinstance(data, dict):
            data = list(data.items())
        typer.echo(tabulate(data, headers=self.headers or tuple()))

    def output_json(self):
        data = self.data
        data = json.dumps(data, indent=2, default=lambda v: str(v))
        typer.echo(data)


output = Output



from typing import Any
from typing import Dict
from typing import List

from pydantic import BaseModel
from pydantic import Field

from fibery.client.base import BaseResource


class FiberyField(BaseModel):
    id: str = Field(alias="fibery/id")
    name: str = Field(alias="fibery/name")
    type: str = Field(alias="fibery/type")
    meta: Dict[str, Any] = Field(alias="fibery/meta")


class FiberyType(BaseModel):
    id: str = Field(alias="fibery/id")
    name: str = Field(alias="fibery/name")
    fields: List[FiberyField] = Field(alias="fibery/fields")
    meta: Dict[str, Any] = Field(alias="fibery/meta")

    def get_field(self, name) -> FiberyField:
        for f in self.fields:
            if f.name.strip() == name.strip():
                return f

        choices = ", ".join([f"'{f.name}'" for f in self.fields])
        raise Exception(f"Field '{name}' does not exist, choices are: {choices}")


class FiberyApplication(BaseModel):
    name: str
    types: List[FiberyType]


class FiberySchema(BaseModel):
    id: str = Field(alias="fibery/id")
    types: List[FiberyType] = Field(None, alias="fibery/types")
    meta: Dict[str, Any] = Field(alias="fibery/meta")

    def list_applications(self) -> List[FiberyApplication]:
        result = {}
        for t in self.types:
            app_name, type_name = t.name.split("/", 1)
            if app_name not in result:
                result[app_name] = FiberyApplication(name=app_name, types=[t])
            else:
                result[app_name].types.append(t)
        return list(sorted(result.values(), key=lambda a: a.name.lower()))

    def has_application(self, application):
        result = set()
        for t in self.types:
            app_name, type_name = t.name.split("/", 1)
            result.add(app_name)
        return application in result

    def list_types(self, application=None):
        result = []
        all_applications = self.list_applications()
        application_choices = ", ".join([f"'{app.name}'" for app in all_applications])
        if application and not self.has_application(application):
            raise Exception(f"Application '{application}' does not exist, choices are: {application_choices}")

        for fibery_type in sorted(self.types, key=lambda v: v.name.lower()):
            type_application, table = fibery_type.name.split("/", 1)
            if application and application != type_application:
                continue
            result.append(fibery_type)
        return result

    def get_type(self, name) -> FiberyType:
        for t in self.types:
            if t.name.strip() == name.strip():
                return t

        choices = ", ".join([f"'{t.name}'" for t in self.types])
        raise Exception(f"Type '{name}' does not exist, choices are: {choices}")


class Schema(BaseResource):

    name = "fibery.schema/query"

    def get(self) -> FiberySchema:
        return self.client.command(self.name, coerce_to=FiberySchema.parse_obj)



import functools

from typing import List

import typer

from click import BadArgumentUsage

from fibery.client.fibery import Fibery
from fibery.conf import settings
from fibery.console.commands.base import output
from fibery.console.commands.utils import default_client
from fibery.enums import FormatEnum


api = typer.Typer()


def init_fibery(f):
    @functools.wraps(f)
    def wrapper(resource, method, arg: List[str], *args, **kwargs):
        fibery = Fibery(client=default_client())

        try:
            resource = getattr(fibery, resource)
        except AttributeError:
            choices = fibery.get_resources()
            raise BadArgumentUsage(
                f"Fibery '{resource}' resource type does not exist, choices are {', '.join(choices)}"
            )

        try:
            method = getattr(resource, method)
        except AttributeError:
            choices = resource.get_methods()
            raise BadArgumentUsage(f"Resource '{method}' method does not exist, choices are {', '.join(choices)}")

        call_args = dict()
        for a in arg:
            if a.count("=") != 1:
                raise BadArgumentUsage("Arg value should be in format key=value")
            key, value = a.split("=")
            call_args.update({key: value})

        return f(resource, method, call_args, *args, **kwargs)

    return wrapper


KEY_VALUE_PATTERN = r"[^=]+\=.+"


@api.command(name="call")
@init_fibery
def call(
    resource: str,
    method: str,
    arg: List[str] = None,
    format: FormatEnum = FormatEnum.json,
):
    settings.output_format = format
    output(method(**arg)).echo()



import typer

from fibery.conf import settings as fibery_settings
from fibery.console.commands.base import output


settings = typer.Typer()


@settings.command(name="show")
def settings_get():
    output(fibery_settings, headers=["name", "value"]).echo()



from fibery.client.base import BaseClient
from fibery.client.schema import Schema
from fibery.client.webhook import WebHook


class Fibery:

    client: BaseClient = None

    def __init__(self, client: BaseClient):
        self.client = client

    @property
    def webhook(self) -> WebHook:
        return WebHook(self)

    @property
    def schema(self) -> Schema:
        return Schema(self)

    def get_resources(self):
        return [c for c in dir(self) if isinstance(getattr(self, c), WebHook)]



from enum import Enum
from typing import Any
from typing import Dict
from typing import List
from typing import Literal
from typing import Union

from pydantic import AnyUrl
from pydantic import BaseModel
from pydantic import Field
from pydantic import SecretStr
from pydantic.typing import ForwardRef

from fibery.client.base import BaseResource
from fibery.models import FiberyEntityType


class QueryResult:
    pass


class Expression(BaseModel):
    pass


class Operator(str, Enum):
    EQ = "="
    NEQ = "!="
    LT = "<"
    LTE = "<="
    GT = ">"
    GTE = ">="
    CONTAINS = "q/contains"
    NCONTAINS = "q/not-contains"
    IN = "q/in"
    NIN = "q/not-in"
    AND = "q/and"
    OR = "q/or"


def field(*path):
    return path


def eq(lft, rgt):
    return Operator.EQ, lft, rgt


def neq(lft, rgt):
    return Operator.NEQ, lft, rgt


def lt(lft, rgt):
    return Operator.LT, lft, rgt


def lte(lft, rgt):
    return Operator.LTE, lft, rgt


def gt(lft, rgt):
    return Operator.GTE, lft, rgt


def gte(lft, rgt):
    return Operator.GTE, lft, rgt


def contains(lft, rgt):
    return Operator.CONTAINS, lft, rgt


def ncontains(lft, rgt):
    return Operator.NCONTAINS, lft, rgt


def In(lft, rgt):
    return Operator.IN, lft, rgt


def nin(lft, rgt):
    return Operator.NEQ, lft, rgt


class Query(BaseModel):
    type: FiberyEntityType = Field(alias="q/from")
    select: List[Union[str, "Query"]] = Field(alias="q/select")
    where: List[Expression] = Field(alias="q/where")
    order: List[Union[str]] = Field(alias="q/order-by")
    limit: List[Union[str]] = Field(alias="q/limit")


Query.update_forward_refs()


class WebHook(BaseResource):

    name = "fibery.entity/query2"

    def query(self, query: Query, **params) -> QueryResult:
        return self.client.command(self.name, args={"query": query.dict()})



from typing import Any
from typing import Dict
from typing import List
from typing import Literal

from pydantic import BaseModel


FiberyEntityType = str


class BaseEffect(BaseModel):
    id: str
    type: FiberyEntityType


class FiberyEntityCreateEffect(BaseEffect):
    effect: Literal["fibery.entity/update"] = "fibery.entity/create"
    values: Dict[str, Any]


class FiberyEntityUpdateEffect(BaseEffect):
    effect: Literal["fibery.entity/update"] = "fibery.entity/update"
    values: Dict[str, Any]
    valuesBefore: Dict[str, Any]


class FiberyEntityDeleteEffect(BaseEffect):
    effect: Literal["fibery.entity/update"] = "fibery.entity/delete"
    valuesBefore: Dict[str, Any]


class FiberyAddCollectionItemsEffect(BaseEffect):
    effect: Literal["fibery.entity/add-collection-items"] = "fibery.entity/add-collection-items"
    field: str
    items: List[Dict[str, Any]]


class FiberyRemoveCollectionItemsEffect(BaseEffect):
    effect: Literal["fibery.entity/add-collection-items"] = "fibery.entity/remove-collection-items"
    field: str
    items: List[Dict[str, Any]]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fibery', 'fibery.client', 'fibery.console', 'fibery.console.commands']

package_data = \
{'': ['*'], 'fibery.console.commands': ['js/*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'aiohttp>=3.7.4,<4.0.0',
 'colorama>=0.4.4,<0.5.0',
 'pydantic>=1.8.1,<2.0.0',
 'pyppeteer>=0.2.5,<0.3.0',
 'requests>=2.25.1,<3.0.0',
 'tabulate>=0.8.9,<0.9.0',
 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['fibery = fibery.console.main:main']}

setup_kwargs = {
    'name': 'fibery',
    'version': '0.4.1',
    'description': 'Fibery.io Python SDK',
    'long_description': None,
    'author': 'Sergio Bershadsky',
    'author_email': 'sergio.bershadsky@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/sergio-bershadsky/fibery',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
